class Provider(object):
    """
    socket.AF_INET addresses provider.
    """

    def __init__(self, host, port):
        """
        :param host:
        :type: str
        :param port:
        :type: int
        """
        self._host = host
        self._port = port

    def get(self):
        """
        :rtype: tuple
        """
        return (self._host, self._port)



from datetime import datetime, timedelta
import logging

import srvlookup


logger = logging.getLogger(__name__)


class Provider(object):
    """
    socket.AF_INET addresses provider based on SRV records
    https://en.wikipedia.org/wiki/SRV_record
    """

    PROTO_TCP = 'TCP'
    PROTO_UDP = 'UDP'

    def __init__(self, service, domain, protocol=None, refresh_delta=None):
        """
        :param service:
        :type: str
        :param domain:
        :type: str
        :param protocol: PROTO_TCP or PROTO_UDP
        :type: str
        :param refresh_delta: interval between refreshing SRV record by
                              talking to DNS server.
        :type: datetime.timedelta
        """
        self._service = service
        self._domain = domain
        self._protocol = self.PROTO_UDP if protocol is None else protocol
        self._refresh_delta = (timedelta(minutes=1) if refresh_delta is None
                               else refresh_delta)
        self._last_refresh_at = datetime.fromtimestamp(0)
        self._addr = None

    def _refresh_addr(self):
        """
        Gets current list of SRV records from DNS server and use the first one.

        TODO: Check if we can assume that server returns records in random order.
        """
        try:
            srvs = srvlookup.lookup(self._service, self._protocol,
                                    self._domain)
        except srvlookup.SRVQueryFailure:
            logger.debug('SRV query failed')
            return

        assert len(srvs) > 0
        srv = srvs[0]
        self._addr = (srv.host, srv.port)
        self._last_refresh_at = datetime.now()

    def get(self):
        """
        :rtype: tuple or None
        """
        if (datetime.now() - self._last_refresh_at) > self._refresh_delta:
            self._refresh_addr()

        return self._addr



import logging
import socket


logger = logging.getLogger(__name__)


class Client(object):
    """
    StatsD UDP client supporting tags (Telegraf's extension)
    """

    def __init__(self, address_provider, tags=None):
        """
        :param address_provider: socket.AF_INET addresses provider
        :type: object
        :param tags: tags to add to each metric
        :type: dict
        """
        self._address_provider = address_provider
        self._tags = tags

    def send(self, payload):
        """
        :param payload: actual data to send
        :type: str
        """
        soc = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        address = self._address_provider.get()

        if address is None:
            logger.error('Address not provided',
                         extra={'address_provider': self._address_provider})
            return

        soc.sendto(payload, address)

    def incr(self, name, delta=1, tags=None):
        """
        :param name:
        :type: str
        :param delta:
        :type: int
        :param tags:
        :type: dict
        """
        name = self._append_tags_to_name(name, tags)
        payload = '{0}:{1}|c'.format(name, delta)
        self.send(payload)

    def _append_tags_to_name(self, name, tags):
        """
        :param name: metric's name
        :type: str
        :param tags: tags to append
        :type: dict
        :rtype: str
        """
        merged_tags = self._tags.copy()

        if tags is not None:
            merged_tags.update(tags)

        serialized_tags = ','.join(['%s=%s' % (k, v) for k, v in
                                    merged_tags.iteritems()])

        if serialized_tags:
            return name + ',' + serialized_tags
        else:
            return name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

from telestatsd import __version__


setup(name='telestatsd',
      version=__version__,
      description=('StatsD client in Python with support for Telegraf '
                   'and SRV records in DNS'),
      author='Opera Wrocław Services',
      author_email='svc-code@opera.com',
      url='https://github.com/operasoftware/telestatsd/',
      license='MIT',
      keywords='statsd telegraf srv service record',
      download_url=('https://github.com/operasoftware/telestatsd/tarball/' +
                    __version__),
      packages=find_packages(),
      install_requires=['srvlookup==0.2.0'],
      zip_safe=False)



import logging
import socket


logger = logging.getLogger(__name__)


class Client(object):
    """
    StatsD client dropping all metrics.
    """

    def incr(self, name, delta=1, tags=None):
        pass
